"""Test SSOClientUtils."""

import pytest
from discourse_django_sso.views import InMemoryNonceService, NonceService
from discourse_django_sso.utils import SSOClientUtils


# pylint: disable=missing-docstring,redefined-outer-name,invalid-name


@pytest.fixture()
def sso_client(
        example_secret,
        example_sso_url,
):
    return SSOClientUtils(
        sso_key=example_secret,
        base_sso_url=example_sso_url,
    )


def test_get_sso_redirect(sso_client: SSOClientUtils):
    # expected = 'http://www.example.com/discourse/sso?' \
    #            'sso=bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGI%3D%0A&' \
    #            'sig=2828aa29899722b35a2f191d34ef9b3ce695e0e6eeec47deb46d588d70c7cb56'
    expected = 'http://www.example.com/discourse/sso?' \
               'sso=bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGI%3D&' \
               'sig=1ce1494f94484b6f6a092be9b15ccc1cdafb1f8460a3838fbb0e0883c4390471'
    actual = sso_client.generate_sso_url(b'cb68251eefb5211e58c00ff1395f0c0b', False)
    assert expected == actual


def test_sso_redirect_activation_req(sso_client: SSOClientUtils):
    """
    Should generate redirect url for sso when require_activation is true
    :param sso_client:
    :return:
    """
    expected = 'http://www.example.com/discourse/sso?' \
               'sso=bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGImcmVxdWlyZV9hY3RpdmF0aW' \
               '9uPXRydWU%3D&' \
               'sig=d6fe1e49e146c1487d06fb8acfb6ed77a31575e715d18daaa19ff8f2f43576d8'
    actual = sso_client.generate_sso_url(b'cb68251eefb5211e58c00ff1395f0c0b', True)
    assert expected == actual


def test_decode_sso_data(sso_client: SSOClientUtils):
    user_data = sso_client.decode_client_data(
        b'bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGImbmFtZT1zYW0mdXNlcm5hbWU9\nc2Ftc2FtJm'
        b'VtYWlsPXRlc3QlNDB0ZXN0LmNvbSZleHRlcm5hbF9pZD1oZWxsbzEyMyZyZXF1aXJl\n'
        b'X2FjdGl2YXRpb249dHJ1ZQ==\n')
    print('MIK' + sso_client.get_param(user_data, 'nonce'))
    assert sso_client.get_param(user_data, b'nonce') == 'cb68251eefb5211e58c00ff1395f0c0b'


def test_sid_sso_validation(sso_client: SSOClientUtils):
    assert sso_client.validate_sso_against_sid(
        b'bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGImbmFtZT1zYW0mdXNlcm5hbWU9\nc2Ftc2FtJm'
        b'VtYWlsPXRlc3QlNDB0ZXN0LmNvbSZleHRlcm5hbF9pZD1oZWxsbzEyMyZyZXF1aXJl\nX2FjdGl2YXRpb249dHJ1'
        b'ZQ==\n', b'3a8dd1a73254003d616d610f66049cf741dfcb924c76b9e75efa01b2507ad0d0') is True


@pytest.fixture()
def nonce_service():
    return InMemoryNonceService()


def test_generate_nonce(nonce_service: NonceService):
    nonce = nonce_service.generate_nonce()
    assert nonce is not None


def test_invalidate_nonce(nonce_service: NonceService):
    nonce = nonce_service.generate_nonce()
    assert nonce_service.is_nonce_valid(nonce) is True
    nonce_service.invalidate_nonce(nonce)
    assert nonce_service.is_nonce_valid(nonce) is False


def test_nonce_valid_for_not_existing_nonce(nonce_service: NonceService):
    assert nonce_service.is_nonce_valid('NOT_GENERATED_VALUE') is False



# coding=utf-8

"""Test sso view."""

import base64
import hashlib
import hmac
from urllib.parse import urlencode

import pytest
from django.test.client import Client
from django.urls.base import reverse
from django.utils.encoding import force_bytes

from discourse_django_sso_test_project import settings


# pylint: disable=missing-docstring,redefined-outer-name,invalid-name


@pytest.fixture()
def nonce():
    return "random_nonce"


@pytest.fixture()
def sso_key():
    return force_bytes(settings.DISCOURSE_SSO_KEY)


@pytest.fixture()
def sso_contents(nonce, sso_key):
    sso = base64.b64encode(urlencode({'nonce': nonce}).encode('ascii')) + b'\n'
    sig = force_bytes(hmac.new(sso_key, force_bytes(sso), digestmod=hashlib.sha256).hexdigest())
    return {
        "sig": sig,
        "sso": sso
    }


def test_view(admin_client: Client, sso_contents):
    response = admin_client.get(reverse('sso'), data=sso_contents)
    assert response.status_code == 302
    expected_response = (
        "http://localhost/test?"
        "sso=bm9uY2U9cmFuZG9tX25vbmNlJmVtYWlsPWFkbWluJTQwZXhhbXBsZS5jb20mdXNlcm5hbWU9YWRtaW4mZXh0Z"
        "XJuYWxfaWQ9MQ%3D%3D&"
        "sig=8fdf24135101a0e9c360a93ca8c455c666e81bd3b4468aa9d06e1da652cb328a"
    )
    assert response.url == expected_response


def test_view_empty_get(admin_client: Client):
    response = admin_client.get(reverse('sso'), data={})
    assert response.status_code == 400


def test_invalid_data(admin_client: Client):
    response = admin_client.get(reverse('sso'), data={
        "sso": "bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGI=\n",
        "sig": "invalid_sig"
    })
    assert response.status_code == 400


def test_view_no_newline(admin_client: Client):
    response = admin_client.get(reverse('sso'), data={
        "sso": "bm9uY2U9Y2I2ODI1MWVlZmI1MjExZTU4YzAwZmYxMzk1ZjBjMGI=",
        "sig": "invalid_sig"
    })
    assert response.status_code == 400


def test_client_url_generaion(admin_client: Client):
    response = admin_client.get(reverse('client'), data={})
    assert response.status_code == 302
    expected_response = (
        "http://www.olcms.org/sso?"
        "sso=bm9uY2U9ZTE1OTE3ZTAzMjA5OTI1OTZmNjM4ZWJmZjc2NzgyMDI%3D&"
        "sig=b7e36a9804e525d398d110fac60e3eba7c5f3d115551a261b9468581fa7cbe35"
    )
    assert response.url == expected_response


def test_required_args_for_session_gen(admin_client: Client):
    """
    Check that sig and sso are required parameters.
    :param admin_client:
    :return:
    """
    response = admin_client.get(reverse('createSession'),
                                data={})
    assert response.status_code == 400
    response = admin_client.get(reverse('createSession'),
                                data={"sso": "does not matter"})
    assert response.status_code == 400


def test_error_sso_not_match_sig(admin_client: Client):
    """
    Check that when sso does not match sig error is returned
    :param admin_client:
    :return:
    """
    response = admin_client.get(reverse('createSession'),
                                data={"sso": "does not match", "sig": "not matching"})
    assert response.status_code == 400

# uncomment below lines to generate sso and sig for data that you need for test
# def generate_sig(admin_client: Client):
#  util = SSOClientUtils(sso_key(), 'http://base.url/')  # pylint: disable=wrong-spelling-in-comment
#     payload = [
#         ('nonce', "e15917e0320992596f638ebff7678202"),
#         ('name', "sam"),      # pylint: disable=wrong-spelling-in-comment
#         ('username', "samsam"),   # pylint: disable=wrong-spelling-in-comment
#         ('email', "test@www.pll"),
#         ('external_id', "external_sam"),
#         ('require_activation', "false"),
#     ]
#     signed_stuff = util.prepare_signed_payload(payload)
#     url = util.create_url('http://base.url/', signed_stuff)
#     print(url)
#     assert url is None


def test_session_generation(admin_client: Client):
    response = admin_client.get(reverse('createSession'), data={
        "sso": 'bm9uY2U9ZTE1OTE3ZTAzMjA5OTI1OTZmNjM4ZWJmZjc2NzgyMDImbmFtZT1zYW0mdXNlcm5hbWU9c2Ftc2F'
               'tJmVtYWlsPXRlc3QlNDB3d3cucGxsJmV4dGVybmFsX2lkPWV4dGVybmFsX3NhbSZyZXF1aXJlX2FjdGl2YX'
               'Rpb249ZmFsc2U=',
        "sig": 'a12def3d4e056395742b1f15bee99c52d430f9d868b72c511df8c0f082d9f28e'})
    assert response.status_code == 302



# coding=utf-8

from setuptools import find_packages, setup
import pathlib
import os
from pip.req import parse_requirements
from pip.download import PipSession

MAIN_DIR = pathlib.Path(__file__).absolute().parent


def get_requirements():
  pip_session = PipSession()
  return parse_requirements(
      str(MAIN_DIR / 'requirements' / 'base.in'),
      session=pip_session
  )

packages = find_packages(
  str(MAIN_DIR),
  include=('discourse_django_sso*',),
  exclude=[]
)

# Did I mention that setup.py is not finest piece of software on earth.
# For this to work when installed you'll need to enumerate all template and static file.


def read_dir(package: str, directory: str):
  package_root = os.path.abspath(package.replace(".", "/"))
  directory = os.path.join(package_root, directory)
  res = []
  for root, subFolders, files in os.walk(directory):
    for file in files:
      res.append(
        os.path.relpath(
         os.path.join(root, file),
         package_root
        ))

  return res


if __name__ == "__main__":

  setup(
    name='discourse_django_sso',
    version='0.6.0',
    packages=packages,
    author='Jacek Bzdak',
    author_email='jacek@askesis.pl',
    description='A Discourse sso app for Django',
    install_requires=[str(req.req) for req in get_requirements()],
    package_data={
      package: [] +
        read_dir(package, "static") +
        read_dir(package, "templates")
      for package in packages
    },
    license="Apache 2.0",
    include_package_data=True,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django :: 1.11",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.5"
    ]
  )
